/*

*/

import renderer.scene.*;
import renderer.models.Disk;
import renderer.models.SquareGrid;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;

/**



*/
@SuppressWarnings("serial")
public class Circle_v4_WindowToTheWorld extends Circle_v0_Abstract
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public Circle_v4_WindowToTheWorld()
   {
      super("Renderer 6: Window To The World");

      // Remove the Model that's already in the Scene.
      scene.modelList.remove(0);

      // Create models for a 3x3 grid of circles and squares.
      final Model model1 = new Disk(1.0, 10, 40);
      final Model model2 = new SquareGrid(1.0, 15, 15);
      final Model model3 = new Disk(1.0, 10, 40);
      final Model model4 = new SquareGrid(1.0, 15, 15);
      final Model model5 = new Disk(1.0, 10, 40);
      final Model model6 = new SquareGrid(1.0, 15, 15);
      final Model model7 = new Disk(1.0, 10, 40);
      final Model model8 = new SquareGrid(1.0, 15, 15);
      final Model model9 = new Disk(1.0, 10, 40);

      // Translate eight of the nine models.
      moveModel(model1,  -2,  2, 0);  // upper left-hand corner
      moveModel(model2,   0,  2, 0);  // center of top edge
      moveModel(model3,   2,  2, 0);  // upper right-hand corner
      moveModel(model4,  -2,  0, 0);  // center of left edge
    //moveModel(model5,   0,  0, 0);  // the model at the center
      moveModel(model6,   2,  0, 0);  // center of right edge
      moveModel(model7,  -2, -2, 0);  // lower left-hand corner
      moveModel(model8,   0, -2, 0);  // center of bottom edge
      moveModel(model9,   2, -2, 0);  // lower right-hand corner

      modelToRotate = model5;

      ModelShading.setRandomColor(model1);
      ModelShading.setRandomColor(model2);
      ModelShading.setRandomColor(model3);
      ModelShading.setRandomColor(model4);
      ModelShading.setColor(model5, Color.red); // the model at the center
      ModelShading.setRandomColor(model6);
      ModelShading.setRandomColor(model7);
      ModelShading.setRandomColor(model8);
      ModelShading.setRandomColor(model9);

      // Add all the Models to the Scene.
      scene.addModel(model1, model2, model3,
                     model4, model5, model6,
                     model7, model8, model9);
   }


   /**
      Get in one place the code to set up the viewport and the view volume.
   */
   @Override protected void setupViewing()
   {
      // Get the size of the FrameBuffer.
      final FrameBuffer fb = fbp.getFrameBuffer();
      final int wFB = fb.width;
      final int hFB = fb.height;

      // The viewport is the whole framebuffer.
      fb.setViewport(0, 0, wFB, hFB);

      if (mode == 1)
      {
         // 1. upper left-hand corner
         scene.camera.projOrtho(-1.0,                       // left
                                -1.0 + (2.0 * wFB) / SIZE,  // right
                                 1.0 - (2.0 * hFB) / SIZE,  // bottom
                                 1.0);                      // top
      }
      else if (mode == 2)
      {
         // 2. center of the top edge
         scene.camera.projOrtho(-wFB / (double)SIZE,        // left
                                 wFB / (double)SIZE,        // right
                                 1.0 - (2.0 * hFB) / SIZE,  // bottom
                                 1.0);                      // top
      }
      else if (mode == 3)
      {
         // 3. upper right-hand corner
         scene.camera.projOrtho( 1.0 - (2.0 * wFB) / SIZE,  // left
                                 1.0,                       // right
                                 1.0 - (2.0 * hFB) / SIZE,  // bottom
                                 1.0);                      // top
      }
      else if (mode == 4)
      {
         // 4. center of the right edge.
         scene.camera.projOrtho( 1.0 - (2.0 * wFB) / SIZE,  // left
                                 1.0,                       // right
                                -hFB / (double)SIZE,        // bottom
                                 hFB / (double)SIZE);       // top
      }
      else if (mode == 5)
      {
         // 5. lower right-hand corner
         scene.camera.projOrtho( 1.0 - (2.0 * wFB) / SIZE,  // left
                                 1.0,                       // right
                                -1.0,                       // bottom
                                -1.0 + (2.0 * hFB) / SIZE); // top
      }
      else if (mode == 6)
      {
         // 6. centered of the bottom edge
         scene.camera.projOrtho(-wFB / (double)SIZE,        // left
                                 wFB / (double)SIZE,        // right
                                -1.0,                       // bottom
                                -1.0 + (2.0 * hFB) / SIZE); // top
      }
      else if (mode == 7)
      {
         // 7. lower left-hand corner
         scene.camera.projOrtho(-1.0,                       // left
                                -1.0 + (2.0 * wFB) / SIZE,  // right
                                -1.0,                       // bottom
                                -1.0 + (2.0 * hFB) / SIZE); // top
      }
      else if (mode == 8)
      {
         // 8. centered of the left edge
         scene.camera.projOrtho(-1.0,                       // left
                                -1 + (2.0 * wFB) / SIZE,  // right
                                -hFB / (double)SIZE,      // bottom
                                 hFB / (double)SIZE);     // top
      }
      else if (mode == 9)
      {
         // 9. center of the framebuffer
         scene.camera.projOrtho(-wFB / (double)SIZE,   // left
                                 wFB / (double)SIZE,   // right
                                -hFB / (double)SIZE,   // bottom
                                 hFB / (double)SIZE);  // top
      }

      super.setupViewing();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new Circle_v4_WindowToTheWorld()
      );
   }
}
